/*
 * myregex.h:
 * 
 * simple cover for regular expression class.
 * The class allocates and frees the strings 
 */

#ifndef MYREGEX_H
#define MYREGEX_H


#include <string>
#include <iostream>
#include <assert.h>

using namespace std;

#ifdef HAVE_REGEX_H
extern "C" {
#include <regex.h>
}
#else
#error requires regex.h
#endif

class myregex {
 public:
    static const int REGMAX=40;
    static const int MYREG_GLOB=0x1000000; /* treat as a shell glob */
    string pat;
    regex_t *ncv;
    regmatch_t pmatch[REGMAX];
    string *ret[REGMAX];
    myregex(std::string pat_,int flags){
	this->pat = pat_;
	if(flags & MYREG_GLOB){
	    std::string pat0 = pat;
	    pat = "";
	    for(string::iterator it = pat0.begin();it!=pat0.end();it++){
		if(*it=='?'){ pat+=".";continue;}
		if(*it=='*'){ pat+=".*";continue;}
		pat += *it;
	    }
	    flags = flags & ~MYREG_GLOB; /* remove flag */
	}
	ncv = 0;
	if(pat.size()==0) return;
	memset(ret,0,sizeof(ret));
	ncv = (regex_t *)calloc(sizeof(*ncv),1);
	if(regcomp(ncv,pat.c_str(),flags | REG_EXTENDED)){
	    std::cerr << "regular expression compile error '" << pat << "'\n";
	    exit(1);
	}

    }
    ~myregex(){
	clear();
	if(ncv){
	    regfree(ncv);
	    ncv = 0;
	}
    }
    void clear(){
	if(!ncv) return;
	for(int i=0;i<REGMAX;i++){
	    if(ret[i]){
		delete ret[i];
		ret[i] = 0;
	    }
	}
    }
    int search(std::string line){
	if(!ncv) return 0;
	clear();
	memset(pmatch,0,sizeof(pmatch));
	int r = regexec(ncv,line.c_str(),REGMAX,pmatch,0);
	if(r==REG_NOMATCH) return 0;
	if(r!=0) return 0;		/* some kind of failure */
	/* Make copies of the matches */
	for(int i=0;i<REGMAX && pmatch[i].rm_so != pmatch[i].rm_eo;i++){
	    ret[i] = new std::string(line.substr(pmatch[i].rm_so,pmatch[i].rm_eo-pmatch[i].rm_so));
	}
	return 1;			/* success */
    }
    string *group(int i){
	if(i>=0 && i<REGMAX) return ret[i];
	return 0;
    }
    myregex& operator=(const myregex& other){
	assert(0);			/* not yet implemented */
    }
};

#endif
